# *-* coding:utf-8 *-*
'''Author: Alexey Drozdov(MrNaif)
Company name: Naif Studios
Email: chuff184@gmail.com'''
__author__="Alexey Drozdov(MrNaif)"
__company__="Naif Studios"
__email__="chuff184@gmail.com"
__copyright__=True
try:
    import requests
except ImportError:
    raise ImportError("You must install requests library first!")
try:
    from simplejson import loads as json_loads
except (ImportError,ValueError):
    from json import loads as json_loads
import warnings

class RPCProxy:
    def __init__(self,url,username=None,password=None,xpub=None,verify=True):
        self.url=url
        self.username=username
        self.password=password
        self.xpub=xpub
        self.verify=verify

    def _send_request(self,method,*args,**kwargs):
        if not self.username or not self.password:
            auth=None
        else:
            auth=(self.username,self.password)
        if args:
            arg=args
        elif kwargs:
            arg=kwargs
        else:
            arg=[]
        dict_to_send={"id": 0,"method":method,"params":arg}
        if self.xpub:
            dict_to_send["xpub"]=self.xpub
        response=requests.post(self.url,headers={'content-type': 'application/json'},json=dict_to_send,auth=auth,verify=self.verify)
        response.raise_for_status()
        json=response.json()
        if json["error"]:
            raise ValueError("Error from server: {}".format(json["error"]))
        if json["id"] != 0:
            warnings.warn("ID mismatch!")
        result=json.get("result",{})
        if type(result) == str:
            result=json_loads(result)
        return result

    def __getattr__(self,method,*args,**kwargs):
        def wrapper(*args,**kwargs):
            return self._send_request(method,*args,**kwargs)
        return wrapper



# pylint: disable=import-error, invalid-sequence-index
import sys
sys.path.append("..")
try:
    from ..coin import Coin
except (ValueError,ImportError):
    from coin import Coin

class BTC(Coin):
    coin_name="BTC"
    friendly_name="Bitcoin"
    providers=["jsonrpcrequests"]

    def __init__(self,rpc_url,rpc_user=None,rpc_pass=None,xpub=None):
        super().__init__()
        self.rpc_url = rpc_url
        self.rpc_user = rpc_user
        self.rpc_pass = rpc_pass
        self.xpub=xpub
        self.server = self.providers["jsonrpcrequests"].RPCProxy(self.rpc_url,self.rpc_user,self.rpc_pass,self.xpub)

    def get_tx(self, tx: str) -> dict:
        out=self.server.get_transaction(tx)
        try:
            out["input"]=out["inputs"][0]["address"]
        except (KeyError,IndexError):
            out["input"]=None
        return out

    def get_address(self, address: str) -> list:
        out = self.server.getaddresshistory(address)
        for i in out:
            i["tx"] = self.get_tx(i["tx_hash"])
        return out

    def balance(self) -> dict:
        data = self.server.getbalance()
        return {"confirmed": data.get("confirmed"),
            "unconfirmed": data.get("unconfirmed"),
            "unmatured": data.get("unmatured")}

    def addrequest(self, amount, description="", expire=15) -> dict:
        return self.server.addrequest(amount=amount, memo=description, expiration=60*expire, force=True)
        
    def getrequest(self, address: str) -> dict:
        return self.server.getrequest(address)

    def history(self) -> list:
        return self.server.history()



import sys
import importlib

SYSTEM_PACKAGES=["electrum"]

class Coin:
    """Coins should reimplement some methods,
    and initialize coin-specific info.
    Required information is:
    coin_name str
    friendly_name str
    providers list
    For more info see the docs.
    """

    coin_name="Base"
    friendly_name="Base"
    providers=[]

    def __init__(self):
        # Initialize the providers.
        self.providers_new = {}
        for i in self.providers:
            if i not in SYSTEM_PACKAGES:
                self.providers_new[i] = importlib.import_module(".providers."+i,"bitcart")
            else:
                self.providers_new[i] = importlib.import_module(i)
        self.providers = self.providers_new
        del self.providers_new

    def get_tx(self, tx: str) -> dict:
        """Override this method for getting transaction info,
        leave the function signature the same.
        For example of required output see the docs
        """
        raise NotImplementedError()

    def get_address(self, address: str) -> list:
        """Override this method for getting address info,
        leave the function signature the same.
        For example of required output see the docs
        """
        raise NotImplementedError()

    def balance(self) -> dict:
        """Override this method to return dict of balances,
        exactly like this:
        >>> self.balance()
        {"confirmed": "0.00005", "unconfirmed": None, "unmatured": None}
        """
        raise NotImplementedError()



from setuptools import setup, find_packages
setup(
  name = 'bitcart',  
  packages = find_packages(),
  version = '0.1.1',    
  license='MIT',       
  description = open("README.md").read(),
  author = 'MrNaif2018',               
  author_email = 'chuff184@gmail.com',      
  url = 'https://github.com/MrNaif2018/bitcart-sdk/',  
  download_url = 'https://github.com/MrNaif2018/bitcart-sdk/archive/0.1.tar.gz',   
  keywords = ['electrum',' daemon', 'bitcart'], 
  install_requires=[        
          'requests',
          'simplejson'
      ],
  classifiers=[
    'Development Status :: 3 - Alpha',   

    'Intended Audience :: Developers',     
    'Topic :: Software Development :: Build Tools',

    'License :: OSI Approved :: MIT License',

    'Programming Language :: Python :: 3',   
    'Programming Language :: Python :: 3.4',
    'Programming Language :: Python :: 3.5',
    'Programming Language :: Python :: 3.6',
    'Programming Language :: Python :: 3.7',
    'Programming Language :: Python :: 3.8',
  ],
)
